2016-03-03


So, again, we have to make a loader.
This is maybe the 50th loader that I've made, 
but it seems that every time I need to create a loader, I start from scratch.
 
Enough.
I was dumb, but not now anymore.


Please give me a simple oneliner for a loader.
I'm using jquery most of the time.



The Why/When
------------

Okay.
So why/when do you need a loader in the first place.
Most of the time, it's because there is a server request (the only? source of latency in 
your otherwise beautiful js gui) at some point.

So if you are using jquery, this code might be familiar:

```js
    $(document).ready(function () {


        var jTarget = $('#target'); // some target container

        $('#click').on('click', function () { // some events being triggered
        
        
            // and now the server request
            $.post('/service/service.php', function (content) {
            
                    // and finally, when the request returns, we do something with the content
                    jTarget.html(content);
                })
                .always(function () {
                    console.log('always');
                });
                
                
                
        });

    });
```


Most of the time, what will you do with the server content: inject it somehow in your page.
So the code above is our working pattern in this discussion.




Proposal
----------------


```js
    $(document).ready(function () {


        var jTarget = $('#target'); // some target container

        $('#click').on('click', function () { // some events being triggered
        
        
        
            jTarget.ajaxloader(); // start the loader
        
        
            // and now the server request
            $.post('/service/service.php', function (content) {
            
                    // and finally, when the request returns, we do something with the content
                    jTarget.html(content);
                })
                .always(function () {
                    jTarget.ajaxloader("stop");
                });
                
                
                
        });

    });
```




Ok, that seems logical, but what about the appearance of the loader?



Appearance
------------

I propose a simple tool that handles THIS (assumed most common cases) case well, 
rather than a complex tool that handles ALL cases.

So, the simplest approach, I believe, to inject a loader is to prepend once 
an absolute positioned div.loader (to the target).

Why absolute?
Because then we can paint it over the other layers behind, using z-index.

The only downside that I found is that due to css rules, 
the host (target) must be positioned (position: not static) in order for this to work.

But that's okay with me.
Now let's continue the conception.


So the appearance of a given target, once ajaxloaderified, will look like this:

```html 
<target> 
    <loader_overlay>
        <loader/>
    </loader_overlay>
    the target content...
</target> 
```
 
 
That would be the base of all the styling.
Now, to make things interesting, the ajaxloader should provide different loader types:
it should be able to handle a provided gif, or to take advantage of css animations as well,
depending on the dev humour.


By convention, overlay has only one direct child
---------------
The loader overlay would just be a high z-index layer that hold its only element: 
the loader.
The loader element might be an image, or a div, or something else, but should always be 
the only direct children of the loader overlay; that's because it then makes it easier
to center the loader in the overlay.

Also, by following this convention, designers have a consistent base to start with.


In the demo, I will provide a gif example, and a css transformations example.
Styles will grow up as needed.
Pull requests will be welcome.




Api proposal
---------

Now to style the loader, one needs to call the correct external stylesheet first, 
and then add the correct class on the loader overlay, or loader overlay parent (the target).

Since the dev has access to the html markup, one of the most handy ways to call a style is 
simply to add the css class on the target:

```html
<div class="target ajaxloader_red_flashy">
</div>
```


However, for the sake of flexibility, dev should also be able to use the api: 


```js 
jTarget.ajaxloader({cssClass: "ajaxloader_red_flashy"}); // start the loader
``` 

Which would result in a different but similar markup:

```html
<div class="target">
    <div class="loader_overlay ajaxloader_red_flashy">
        <div class="loader"/>
    </div>
</div>
```

That's to emphasize the fact that the loader element has always an ancestor that
carries the style that should apply to it.
So css stylesheets should have rules that starts with a style class, and end with the loader class 
like this:


```css
.ajaxloader_red_flashy .loader{
    // the red flashy loader style...
}
```


Also, if the dev doesn't want to use a particular style, but has an image nearby, 
she should be able to use it with:


```js 
jTarget.ajaxloader({img: "/url/to/image.gif"}); // start the loader
``` 

Which would result in something like the following: 


```html
<div class="target">
    <div class="loader_overlay">
        <img src="/url/to/image.gif" class="loader"/>
    </div>
</div>
```



The default call
------------------

The default call to the ajaxloader method should invoke the loader and load a default image.






















 










